@@ -211,9 +211,17 @@ def Node_getAttributeNode(node, name):
 def Node_setAttributeNS(node, ns, name, value):
     ns, name, value = map(from_unicode, [ns, name, value])
     prefix, localName = _get_prefix_and_localName(name)
-    if ns is not None:
-        if prefix == "xmlns" and ns == xml.dom.XMLNS_NAMESPACE and _find_namespace(node, value, localName):
+
+    # Detect setting of xmlns:localName=value, looking for cases where
+    # x:attr=value have caused the definition of xmlns:x=y (as a declaration
+    # with prefix=x, ns=y).
+    if prefix == "xmlns" and ns == xml.dom.XMLNS_NAMESPACE:
+        if _find_namespace(node, value, localName):
             return
+        new_ns = _make_namespace(node, value, localName, set_default=0)
+    # For non-xmlns attributes, we find or make a namespace declaration and then
+    # set an attribute.
+    elif ns is not None:
         new_ns = _find_namespace(node, ns, prefix)
         if new_ns is None:
             new_ns = _make_namespace(node, ns, prefix, set_default=0)
